import os
import requests

def load_data(filename):
    """
    function to load data with family codes from csv to dataframe

    :param filename: name of csv file with columns with accession numbers with header
    :type filename: str
    :return: list with data from the file
    :rtype: list
    """
    file = open(filename)
    list = file.readlines()
    try:
        return list
    except FileNotFoundError:
        print('No stats file found')


def get_names(list):
    """
    function to save names from data frame to list

    :param list: list with first column of pfam accession numbers
    :type list: list
    :return: list of accession numbers
    :rtype: list
    """
    family_names = []
    for i in range(1,len(list)):
        splitedlist=list[i].split(",")
        family_names += [splitedlist[0]]
    if len(family_names) == 0:
        raise ValueError('No families for this number of sequences')
    return family_names


def download_hmm(families, dir):
    """
    function to download hmm profiles of families which names are in the list

    :param families: list of families
    :type families: list
    :param dir: name of directory to which data are downloaded
    :type dir: str
    :return: downloaded hmm files from pfam
    :rtype: file
    """
    global type
    for family in families:
        url = 'http://pfam.xfam.org/family/###FAMILY_NAME###/hmm'
        url = url.replace('###FAMILY_NAME###', family)
        r = requests.get(url, allow_redirects=True) # download hmm
        r.raise_for_status()
        pathname = dir + "/" + family + '.hmm'
        if not os.path.exists('./hmm_folder'):
            os.makedirs('./hmm_folder')
        f = open(pathname, 'wb').write(r.content)




import os

"""
module for wrapping HMMER tools: hmmsearch, hmmpress and hmmscan  
"""
def hmmsearch(path="hmmsearch", h=False, o='out', A=False, tblout=False, domtblout=False, pfamtblout=False, acc=False, noali=False, notextw=False, textw=False,
                         E=False, T=False, domE=False, domT=False, incE=False, incT=False, incdomE=False, incdomT=False, cut_ga=False, cut_nc=False, cut_tc=False,
                         max=False, F1=False, F2=False, F3=False, nobias=False, nonull=False, Z=False, domZ=False, seed=False, tformat=False, cpu=False, hmm_file=False,
                         fasta_file=False):
    """
    Wrapper for HMMsearch tool from HAMMER (description from HAMMER manual)

    :param path:  path to hmmscan tool
    :type path: str
    :param h: Help; print a brief reminder of command line usage and all available options - optional
    :type h: bool
    :param o: direct output to file <f>, not stdout
    :type o:  str
    :param A: save multiple alignment of all hits to file <f>
    :type A:  str, optional
    :param tblout: save parseable table of per-sequence hits to file <f>
    :type tblout:  str, optional
    :param domtblout: save parseable table of per-domain hits to file <f>
    :type domtblout: str, optional
    :param pfamtblout: save table of hits and domains to file, in Pfam format <f>
    :type pfamtblout: str, optional
    :param acc: prefer accessions over names in output - optional
    :type acc: bool
    :param noali: don't output alignments, so output is smaller - optional
    :type noali: bool
    :param notextw: unlimit ASCII text output line width - optional
    :type notextw: bool
    :param textw: set max width of ASCII text output lines  [120]  (n>=120)
    :type textw: int, optional
    :param E: report sequences <= this E-value threshold in output  [10.0]  (x>0)
    :type E: float, optional
    :param T: report sequences >= this score threshold in output
    :type T: float, optional
    :param domE: report domains <= this E-value threshold in output  [10.0]  (x>0)
    :type domE: float, optional
    :param domT: report domains >= this score cutoff in output
    :type domT: float, optional
    :param incE: consider sequences <= this E-value threshold as significant
    :type incE: float, optional
    :param incT:  consider sequences >= this score threshold as significant
    :type incT: float, optional
    :param incdomE: consider domains <= this E-value threshold as significant
    :type incdomE: float, optional
    :param incdomT: consider domains >= this score threshold as significant
    :type incdomT: float, optional
    :param cut_ga: use profile's GA gathering cutoffs to set all thresholding - optional
    :type cut_ga: bool
    :param cut_nc: use profile's NC noise cutoffs to set all thresholding -  optional
    :type cut_nc: bool
    :param cut_tc: use profile's TC trusted cutoffs to set all thresholding - optional
    :type cut_tc: bool
    :param max: Turn all heuristic filters off (less speed, more power) - optional
    :type max: bool
    :param F1: Stage 1 (MSV) threshold: promote hits w/ P <= F1  [0.02]
    :type F1: float, optional
    :param F2: Stage 2 (Vit) threshold: promote hits w/ P <= F2  [1e-3]
    :type F2: float, optional
    :param F3: Stage 3 (Fwd) threshold: promote hits w/ P <= F3  [1e-5]
    :type F3: float, optional
    :param nobias: turn off composition bias filter - optional
    :type nobias: bool
    :param nonull: turn off biased composition score corrections - optional
    :type nonull: bool
    :param Z: set # of comparisons done, for E-value calculation
    :type Z: float, optional
    :param domZ: set # of significant seqs, for domain E-value calculation
    :type domZ: float, optional
    :param seed: set RNG seed to <n> (if 0: one-time arbitrary seed)  [42]
    :type seed: int, optional
    :param tformat: assert target <seqfile> is in format <s>: no autodetection
    :type tformat: str, optional
    :param cpu: number of parallel CPU workers to use for multithreads  [1]
    :type cpu: int, optional
    :param hmm_file:  directory of hmm file
    :type hmm_file: str
    :param fasta_file: directory of fasta file
    :type fasta_file: str
    :return: output file from hmmsearch
    :rtype: file
    """

    bashCommand_hmm_search=path+" "
    if h:
        if isinstance(h, bool):
            bashCommand_hmm_search+="-h"
        else:
            raise TypeError
    else:
        if o:
            if isinstance(o, str):
                bashCommand_hmm_search += "-o " + o + " "
            else:
                raise TypeError
        if A:
            if isinstance(A, str):
                bashCommand_hmm_search += "-A " + A + " "
            else:
                raise TypeError

        if tblout:
            if isinstance(tblout, str):
                bashCommand_hmm_search += "--tblout " + tblout + " "
            else:
                raise TypeError
        if domtblout:
            if isinstance(domtblout, str):
                bashCommand_hmm_search += "--domtblout " + domtblout + " "
            else:
                raise TypeError
        if pfamtblout:
            if isinstance(pfamtblout, str):
                bashCommand_hmm_search += "--pfamtblout " + pfamtblout + " "
            else:
                raise TypeError
        if acc:
            if isinstance(acc, bool):
                bashCommand_hmm_search += "--acc "
            else:
                raise TypeError
        if noali:
            if isinstance(noali, bool):
                bashCommand_hmm_search += "--noali "
            else:
                raise TypeError
        if notextw:
            if isinstance(notextw, bool):
                bashCommand_hmm_search += "--notextw "
            else:
                raise TypeError
        if textw:
            if isinstance(notextw, bool):
                bashCommand_hmm_search += "--textw " + textw + " "
            else:
                raise TypeError
        if E:
            if isinstance(E, float):
                bashCommand_hmm_search += "-E " + str(E) + " "
            else:
                raise TypeError
        if T:
            if isinstance(T, float):
                bashCommand_hmm_search += "-T " + str(T) + " "
            else:
                raise TypeError
        if domE:
            if isinstance(domE, float):
                bashCommand_hmm_search += "--domE " + str(domE) + " "
            else:
                raise TypeError
        if domT:
            if isinstance(domT, float):
                bashCommand_hmm_search += "--domT " + str(domT) + " "
            else:
                raise TypeError
        if incE:
            if isinstance(incE, float):
                bashCommand_hmm_search += "--incE " + str(incE) + " "
            else:
                raise TypeError
        if incT:
            if isinstance(incT, float):
                bashCommand_hmm_search += "--incE " + str(incT) + " "
            else:
                raise TypeError
        if incdomE:
            if isinstance(incdomE, float):
                bashCommand_hmm_search += "--incdomE " + str(incdomE) + " "
            else:
                raise TypeError
        if incdomT:
            if isinstance(incdomT, float):
                bashCommand_hmm_search += "--incdomE " + str(incdomT) + " "
            else:
                raise TypeError
        if cut_ga:
            if isinstance(cut_ga, bool):
                bashCommand_hmm_search += "--cut_ga "
            else:
                raise TypeError
        if cut_nc:
            if isinstance(cut_nc, bool):
                bashCommand_hmm_search += "--cut_nc "
            else:
                raise TypeError
        if cut_tc:
            if isinstance(cut_tc, bool):
                bashCommand_hmm_search += "--cut_tc "
            else:
                raise TypeError
        if max:
            if isinstance(max, bool):
                bashCommand_hmm_search += "--max "
            else:
                raise TypeError
        if F1:
            if isinstance(F1, float):
                bashCommand_hmm_search += "--F1 " + str(F1) + " "
            else:
                raise TypeError
        if F2:
            if isinstance(F2, float):
                bashCommand_hmm_search += "--F1 " + str(F2) + " "
            else:
                raise TypeError
        if F3:
            if isinstance(F3, float):
                bashCommand_hmm_search += "--F1 " + str(F3) + " "
            else:
                raise TypeError
        if nobias:
            if isinstance(nobias, bool):
                bashCommand_hmm_search += "--nobias "
            else:
                raise TypeError
        if nonull:
            if isinstance(nonull, bool):
                bashCommand_hmm_search += "--nonull "
            else:
                raise TypeError
        if Z:
            if isinstance(Z, float):
                bashCommand_hmm_search += "-Z " + str(Z) + " "
            else:
                raise TypeError
        if domZ:
            if isinstance(domZ, float):
                bashCommand_hmm_search += "--domZ " + str(domZ) + " "
            else:
                raise TypeError
        if seed:
            if isinstance(seed, int):
                bashCommand_hmm_search += "--seed " + str(seed) + " "
            else:
                raise TypeError
        if tformat:
            if isinstance(tformat, str):
                bashCommand_hmm_search += "--tformat " + str(format) + " "
            else:
                raise TypeError
        if cpu:
            if isinstance(cpu, str):
                bashCommand_hmm_search += "--cpu " + str(cpu) + " "
            else:
                raise TypeError
        if hmm_file:
            if isinstance(hmm_file, str):
                bashCommand_hmm_search += hmm_file + " "
            else:
                raise TypeError
        if fasta_file:
            if isinstance(fasta_file, str):
                bashCommand_hmm_search += fasta_file
            else:
                raise TypeError
    os.system(bashCommand_hmm_search) #run command


def hmmpress(path="hmmpress" ,h=None, f=None, hmm_file=None):
    """
    Wrapper for HMMpress tool from HAMMER (description from HAMMER manual)

    :param path: path to hmmscan tool
    :type path: str
    :param h: Help; print a brief reminder of command line usage and all available options - optional
    :type h: bool
    :param f: Force; overwrites any previous hmmpress’ed datafiles. The default is to bitch about any existing files and ask you to delete them first - optional
    :type f: bool
    :param hmm_file: directory of hmm file
    :type hmm_file: str
    :param fasta_file: directory of fasta file
    :type fasta_file: str
    :return: output from hmmpress - prepared profiles files
    :rtype: file
    """

    bashCommand_hmm_press = path+ " "
    if h:
        if isinstance(h, bool):
            bashCommand_hmm_press += "-h"
        else:
            raise TypeError
    else:
        if f:
            if isinstance(f, bool):
                bashCommand_hmm_press += "-f "
            else:
                raise TypeError
        if hmm_file:
            if isinstance(hmm_file, str):
                bashCommand_hmm_press += hmm_file + " "
            else:
                raise TypeError
    os.system(bashCommand_hmm_press)  # run command


def hmmscan(path="hmmscan", h=False, o='out', tblout=False, domtblout=False, pfamtblout=False, acc=False, noali=False, notextw=False, textw=False,
                         E=False, T=False, domE=False, domT=False, incE=False, incT=False, incdomE=False, incdomT=False, cut_ga=False, cut_nc=False, cut_tc=False,
                         max=False, F1=False, F2=False, F3=False, nobias=False, nonull2=False, Z=False, domZ=False, seed=False, qformat=False, cpu=False, stall=False,
                         MPI=False, hmm_file=False,  fasta_file=False):
    """
    Wrapper for HMMscan tool from HAMMER (description from HAMMER manual)

    :param path: path to hmmscan tool
    :type path: str
    :param h: Help; print a brief reminder of command line usage and all available options- optional
    :type h: bool
    :param o: Direct the main human-readable output to a file <f> instead of the default stdout
    :type o: str
    :param tblout:  Save a simple tabular (space-delimited) file summarizing the per-target output, with one data line per homologous target model found
    :type tblout: str, optional
    :param domtblout: Save a simple tabular (space-delimited) file summarizing the per-domain output, with one data line per homologous domain detected in a query sequence for each homologous model
    :type domtblout: str, optional
    :param pfamtblout: Save an especially succinct tabular (space-delimited) file summarizing the per-target output, with one data line per homologous target model found. hmmer user’s guide 99
    :type pfamtblout: str, optional
    :param acc: Use accessions instead of names in the main output, where available for profiles and/or sequences - optional
    :type acc: bool
    :param noali: Omit the alignment section from the main output. This can greatly reduce the output volume - optional
    :type noali: bool
    :param notextw: Unlimit the length of each line in the main output. The default is a limit of 120 characters per line, which helps in displaying the output cleanly on terminals and in editors, but can truncate target profile description lines - optional
    :type notextw: bool
    :param textw:  Set the main output’s line length limit to <n> characters per line. The default is 120
    :type textw: int, optional
    :param E: In the per-target output, report target profiles with an Evalue of <= <x>. The default is 10.0, meaning that on average, about 10 false positives will be reported per query, so you can see the top of the noise and decide for yourself if it’s really noise
    :type E: float, optional
    :param T: Instead of thresholding per-profile output on E-value, instead report target profiles with a bit score of >= <x>
    :type T: float, optional
    :param domE: In the per-domain output, for target profiles that have already satisfied the per-profile reporting threshold, report individual domains with a conditional E-value of <= <x>
    :type domE: float, optional
    :param domT: Instead of thresholding per-domain output on E-value, instead report domains with a bit score of >= <x>. Options for Inclusion Thresholds
    :type domT: float, optional
    :param incE:  Use an E-value of <= <x> as the per-target inclusion threshold. The default is 0.01, meaning that on average, about 1 100 sean r. eddy false positive would be expected in every 100 searches with different query sequences
    :type incE: float, optional
    :param incT: Instead of using E-values for setting the inclusion threshold, instead use a bit score of >= <x> as the per-target inclusion threshold. It would be unusual to use bit score thresholds with hmmscan, because you don’t expect a single score threshold to work for different profiles; different profiles have slightly different expected score distributions
    :type incT: float, optional
    :param incdomE: Use a conditional E-value of <= <x> as the per-domain inclusion threshold, in targets that have already satisfied the overall per-target inclusion threshold. The default is 0.01.
    :type incdomE: float, optional
    :param incdomT: has been applied specifically using each model’s curated thresholds
    :type incdomE: float, optional
    :param cut_ga: Use the GA (gathering) bit scores in the model to set persequence (GA1) and per-domain (GA2) reporting and inclusion thresholds. GA thresholds are generally considered to be the reliable curated thresholds defining family membership; for example_fasta, in Pfam, these thresholds define what gets included in Pfam Full alignments based on searches with Pfam Seed models - optional
    :type cut_ga: bool
    :param cut_nc: Use the NC (noise cutoff) bit score thresholds in the model to set per-sequence (NC1) and per-domain (NC2) reporting and inclusion thresholds. NC thresholds are generally considered to be the score of the highest-scoring known false positive - optional
    :type cut_nc: bool
    :param cut_tc:  Use the NC (trusted cutoff) bit score thresholds in the model to set per-sequence (TC1) and per-domain (TC2) reporting and inclusion thresholds. TC thresholds are generally considered to be the score of the lowest-scoring known true positive that is above all known false positives. hmmer user’s guide 101 - optional
    :type cut_tc: bool
    :param max: Turn off all filters, including the bias filter, and run full Forward/Backward postprocessing on every target. This increases sensitivity somewhat, at a large cost in speed
    :type max: bool
    :param F1: Stage 1 (MSV) threshold: promote hits w/ P <= F1  [0.02]
    :type F1: float, optional
    :param F2: Stage 2 (Vit) threshold: promote hits w/ P <= F2  [1e-3]
    :type F2: float, optional
    :param F3: Stage 3 (Fwd) threshold: promote hits w/ P <= F3  [1e-5]
    :type F3: float, optional
    :param nobias: Turn off the bias filter. This increases sensitivity somewhat, but can come at a high cost in speed, especially if the query has biased residue composition (such as a repetitive sequence region, or if it is a membrane protein - optional
    :type nobias: bool
    :param nonull2: Turn off the null2 score corrections for biased composition - optional
    :type nonull2: bool
    :param Z: Assert that the total number of targets in your searches is <x>, for the purposes of per-sequence E-value calculations, rather than the actual number of targets seen - optional
    :type Z: float, optional
    :param domZ: Assert that the total number of targets in your searches is <x>, for the purposes of per-domain conditional E-value calculations, rather than the number of targets that passed the reporting thresholds.
    :type domZ: bool
    :param seed: Set the random number seed to <n>. Some steps in postprocessing require Monte Carlo simulation. The default is to use a fixed seed (42), so that results are exactly reproducible. 102 sean r. eddy Any other positive integer will give different (but also reproducible) results. A choice of 0 uses an arbitrarily chosen seed - optional
    :type seed: int, optional
    :param qformat: Assert that input seqfile is in format <s>, bypassing format autodetection. Common choices for <s> include: fasta, embl, genbank. Alignment formats also work; common choices include: stockholm, a2m, afa, psiblast, clustal, phylip. For more information, and for codes for some less common formats, see main documentation. The string <s> is case-insensitive (fasta or FASTA both work).
    :type qformat: str, optional
    :param cpu: Set the number of parallel worker threads to <n>. On multicore machines, the default is 2. You can also control this number by setting an environment variable, HMMER_NCPU. There is also a master thread, so the actual number of threads that HMMER spawns is <n>+1. This option is not available if HMMER was compiled with POSIX threads support turned off
    :type cpu: int, optional
    :param stall: For debugging the MPI master/worker version: pause after start, to enable the developer to attach debuggers to the running master and worker(s) processes. Send SIGCONT signal to release the pause. (Under gdb: (gdb) signal SIGCONT) (Only available if optional MPI support was enabled at compiletime.) - optional
    :type stall: bool
    :param MPI: Run under MPI control with master/worker parallelization (using mpirun, for example_fasta, or equivalent). Only available if optional MPI support was enabled at compile-time. Required argumnets - optional
    :type MPI: bool
    :param hmm_file:  directory of hmm file
    :type hmm_file: str
    :param fasta_file: directory of fasta file
    :type fasta_file: str
    :return: output file from hmmscan
    :rtype: file
    """
    bashCommand_hmm_scan=path+" "
    if h:
        if isinstance(h, bool):
            bashCommand_hmm_scan+="-h"
        else:
            raise TypeError
    else:
        if o:
            if isinstance(o, str):
                bashCommand_hmm_scan += "-o " + o + " "
            else:
                raise TypeError
        if tblout:
            if isinstance(tblout, str):
                bashCommand_hmm_scan += "--tblout " + tblout + " "
            else:
                raise TypeError
        if domtblout:
            if isinstance(domtblout, str):
                bashCommand_hmm_scan += "--domtblout " + domtblout + " "
            else:
                raise TypeError
        if pfamtblout:
            if isinstance(pfamtblout, str):
                bashCommand_hmm_scan += "--pfamtblout " + pfamtblout + " "
            else:
                raise TypeError
        if acc:
            if isinstance(acc, bool):
                bashCommand_hmm_scan += "--acc "
            else:
                raise TypeError
        if noali:
            if isinstance(noali, bool):
                bashCommand_hmm_scan += "--noali "
            else:
                raise TypeError
        if notextw:
            if isinstance(notextw, bool):
                bashCommand_hmm_scan += "--notextw "
            else:
                raise TypeError
        if textw:
            if isinstance(notextw, bool):
                bashCommand_hmm_scan += "--textw " + textw + " "
            else:
                raise TypeError
        if E:
            if isinstance(E, float):
                bashCommand_hmm_scan += "-E " + str(E) + " "
            else:
                raise TypeError
        if T:
            if isinstance(T, float):
                bashCommand_hmm_scan += "-T " + str(T) + " "
            else:
                raise TypeError
        if domE:
            if isinstance(domE, float):
                bashCommand_hmm_scan += "--domE " + str(domE) + " "
            else:
                raise TypeError
        if domT:
            if isinstance(domT, float):
                bashCommand_hmm_scan += "--domT " + str(domT) + " "
            else:
                raise TypeError
        if incE:
            if isinstance(incE, float):
                bashCommand_hmm_scan += "--incE " + str(incE) + " "
            else:
                raise TypeError
        if incT:
            if isinstance(incT, float):
                bashCommand_hmm_scan += "--incE " + str(incT) + " "
            else:
                raise TypeError
        if incdomE:
            if isinstance(incdomE, float):
                bashCommand_hmm_scan += "--incdomE " + str(incdomE) + " "
            else:
                raise TypeError
        if incdomT:
            if isinstance(incdomT, float):
                bashCommand_hmm_scan += "--incdomE " + str(incdomT) + " "
            else:
                raise TypeError
        if cut_ga:
            if isinstance(cut_ga, bool):
                bashCommand_hmm_scan += "--cut_ga "
            else:
                raise TypeError
        if cut_nc:
            if isinstance(cut_nc, bool):
                bashCommand_hmm_scan += "--cut_nc "
            else:
                raise TypeError
        if cut_tc:
            if isinstance(cut_tc, bool):
                bashCommand_hmm_scan += "--cut_tc "
            else:
                raise TypeError
        if max:
            if isinstance(max, bool):
                bashCommand_hmm_scan += "--max "
            else:
                raise TypeError
        if F1:
            if isinstance(F1, float):
                bashCommand_hmm_scan += "--F1 " + str(F1) + " "
            else:
                raise TypeError
        if F2:
            if isinstance(F2, float):
                bashCommand_hmm_scan += "--F1 " + str(F2) + " "
            else:
                raise TypeError
        if F3:
            if isinstance(F3, float):
                bashCommand_hmm_scan += "--F1 " + str(F3) + " "
            else:
                raise TypeError
        if nobias:
            if isinstance(nobias, bool):
                bashCommand_hmm_scan += "--nobias "
            else:
                raise TypeError
        if nonull2:
            if isinstance(nonull2, bool):
                bashCommand_hmm_scan += "--nonull "
            else:
                raise TypeError
        if Z:
            if isinstance(Z, float):
                bashCommand_hmm_scan += "-Z " + str(Z) + " "
            else:
                raise TypeError
        if domZ:
            if isinstance(domZ, float):
                bashCommand_hmm_scan += "--domZ " + str(domZ) + " "
            else:
                raise TypeError
        if seed:
            if isinstance(seed, int):
                bashCommand_hmm_scan += "--seed " + str(seed) + " "
            else:
                raise TypeError
        if qformat:
            if isinstance(qformat, str):
                bashCommand_hmm_scan += "--tformat " + str(format) + " "
            else:
                raise TypeError
        if cpu:
            if isinstance(cpu, str):
                bashCommand_hmm_scan += "--cpu " + str(cpu) + " "
            else:
                raise TypeError

        if stall:
            if isinstance(stall, bool):
                bashCommand_hmm_scan += fasta_file
            else:
                raise TypeError
        if MPI:
            if isinstance(stall, bool):
                bashCommand_hmm_scan += fasta_file
            else:
                raise TypeError
        if hmm_file:
            if isinstance(hmm_file, str):
                bashCommand_hmm_scan += hmm_file + " "
            else:
                raise TypeError
        if fasta_file:
            if isinstance(fasta_file, str):
                bashCommand_hmm_scan += fasta_file
            else:
                raise TypeError
    os.system(bashCommand_hmm_scan) # run command


from .hmmer_command import *
import os

"""
module from automatic search against hmm profiles
"""
def automatic_search(hmm_dir, fasta_dir, out_dir, mode):
    """
    function for automatic search against profiles

    :param hmm_dir: name of directory  with HMMER profiles files
    :type hmm_dir: str
    :param fasta_dir: name of directory with fasta files
    :type fasta_dir: str
    :param out_dir: name of directory for outputs
    :type out_dir: str
    :param mode:  "scan" for hmmscan or "search" for hmmsearch of HMMER tool
    :type mode: str
    :return: outputs for hmmscan or hmmsearch
    :rtype: file
    """
    dir_hmm_list = os.listdir(hmm_dir)
    dir_fasta_list = os.listdir(fasta_dir)
    for fasta_file in dir_fasta_list: #iterate over filees in folder
            fasta_path = fasta_dir + "/" + str(fasta_file)
            if mode == 'scan':
                for hmm_file in dir_hmm_list:
                    #define paths
                    hmm_path = hmm_dir + "/" + str(hmm_file)
                    out_path = out_dir + "/" + str(hmm_file)+str(fasta_file)+"_out"
                    hmmpress(hmm_file=hmm_path) #execute command of hmmpress which prepars files for hmmscan
                    hmmscan(o=out_path, hmm_file=hmm_path, fasta_file=fasta_path) #execute command of  hmmscan
            else:
                for hmm_file in dir_hmm_list: #iterate over files in polder
                    #define paths
                    hmm_path = hmm_dir + "/" + str(hmm_file)
                    out_path = out_dir + "/" + str(hmm_file)+str(fasta_file)+"_out"
                    hmmsearch(o=out_path, hmm_file=hmm_path, fasta_file=fasta_path) #execute command of  hmmsearch




import re
from .hmmer_file import *

def to_dict(tab, mode):
    """
    function to change list to dictionary of emmision (match or insert) -log 0.25 probabilities or transition -log 0.25 probabilities

    :param tab: list of -log 0.25 probabilities from line of file
    :type tab: list
    :param mode: 1 for "match_emission" line of file,  2 for "insert_emission" line of file and 3 for "state_transition" line of file as is the order in HMM block
    :type mode: int
    :return: dictionary of emmision -log 0.25 probabilities and transition -log 0.25 probabilities
    :rtype: dict
    """
    dict = {}
    listaa = ["A", "C", "D", "E", "F", "G", "H", "I", "K", "L", "M", "N", "P", "Q", "R", "S", "T", "V", "W", "Y"] #aa alphabet
    liststate = [("m","m"), ("m","i"), ("m","d"), ("i", "m"), ("i","i"), ("d", "m"), ("d","d")] #all possioble transition states m- match i -insert, d - deletion
    if mode == 1:
        tab = tab[1:22] #save only probability states
    if mode == 2:
        tab = tab[:21] #save only probability states
    if mode == 1 or mode == 2:
        for i in range(len(listaa)):
            aa = listaa[i]
            elt = float(tab[i])
            dict[aa] = elt
    if mode == 0:
        for i in range(len(liststate)):
            state = liststate[i]
            elt = tab[i]
            if elt == "*":
                dict[state] = None
            else:
                dict[state] = float(elt)
    return dict

def file_to_object(filename):
    """
    function which enables to pars HMMER profile file to object

    :param filename:  name or path of HMMER profile file
    :type filename: str
    :return: HMMERProfileFileBuilder object build from file
    :rtype: HMMERProfileFileBuilder object
    """
    match_emission = []
    insert_emission = []
    state_transition = []
    flag=0
    i=0
    version = ""
    name = ""
    acc = ""
    desc = ""
    alph = ""
    rf = ""
    mm = ""
    cons = ""
    cs = ""
    map = ""
    date = ""
    nseq = ""
    effn = ""
    cksum = ""
    ga = ""
    tc = ""
    nc = ""
    bm =""
    sm = ""
    msv = ""
    viterbi = ""
    file = open(filename)
    for line in file: #iterate over lines in file
        if "HMMER" in line:
            version = line.strip()
        if "NAME" in line:
            name = re.findall("[\S]+", line)[1]
        if "ACC" in line:
            acc = re.findall("[\S]+", line)[1]
        if "DESC" in line:
            desc = re.findall("[\S]+", line)[1]
        if "ALPH" in line:
            alph = re.findall("[\S]+", line)[1]
        if "RF" in line:
            if re.findall("[\S]+", line)[1] == "yes":
                rf = True
            else:
                rf = False
        if "MM\t" in line:
            if  re.findall("[\S]+", line)[1] == "yes":
                mm = True
            else:
                mm = False
        if "CONS" in line:
            if re.findall("[\S]+", line)[1]== "yes":
                cons = True
            else:
                cons = False
        if "CS" in line:
            if re.findall("[\S]+", line)[1] == "yes":
                cs = True
            else:
                cs = False
        if "MAP" in line:
            if re.findall("[\S]+", line)[1] == "yes":
                map = True
            else:
                map = False
        if "DATE" in line:
            date = line.split(" ", 2)[2].strip()
        if "NSEQ" in line:
            nseq = int(re.findall("[\S]+", line)[1])
        if "EFFN" in line:
            effn = float(re.findall("[\S]+", line)[1])
        if "CKSUM" in line:
            cksum = int(re.findall("[\S]+", line)[1])
        if  "GA" in line:
            tab = re.findall("[\S]+", line)
            first = float(tab[1])
            secound = float(tab[2].strip(";"))
            ga = [first, secound]
        if  "TC" in line:
            tab = re.findall("[\S]+", line)
            first = float(tab[1])
            secound = float(tab[2].strip(";"))
            tc = [first, secound]
        if  "NC" in line:
            tab = re.findall("[\S]+", line)
            first = float(tab[1])
            secound = float(tab[2].strip(";"))
            nc = [first, secound]
        if "BM" in line:
            bm = line.split(" ",4)
            bm = bm[3].strip()
        if "SM" in line:
            sm = re.findall("[\S]+", line)
            sm = line.split(" ", 1)[1].strip()
        if "MSV" in line:
            tab = re.findall("[\S]+", line)
            first = float(tab[3])
            secound = float(tab[4])
            msv = [first, secound]
        if "VITERBI" in line:
            tab = re.findall("[\S]+", line)
            first = float(tab[3])
            secound = float(tab[4])
            viterbi = [first, secound]
        if "FORWARD" in line:
            tab = re.findall("[\S]+", line)
            first = float(tab[3])
            secound = float(tab[4])
            forward = [first, secound]
        if "COMPO" in line:
            flag = 1
        if "//" in line:
            flag = 0
        if flag == 1:
            i+=1
            linetab = re.findall("[\S]+", line)
            mode=i%3
            if mode == 1:
                match_emission += [to_dict(linetab, mode)]
            if mode == 2:
                insert_emission += [to_dict(linetab, mode)]
            if mode == 0:
                state_transition += [to_dict(linetab, mode)]
    object = HMMERProfileFileBuilder(version, name, acc, desc, alph, rf, mm, cons, cs, map, date, nseq, effn, # use parsed arguments to make an object
               cksum, ga, tc, nc, bm, sm, msv, viterbi, forward, match_emission, insert_emission, state_transition)
    return object





from .pfam_architecture import*
import requests
import re
from urllib.request import urlopen, Request
from urllib.error import HTTPError, URLError
from bs4 import BeautifulSoup

def family_architectures(name,clan=False):
    """ Get domains architectures for given name.
    
    :param name: pfam access key
    :type name: str
    :param clan: Is entry a clan?
    :type clan: bool
    :result: list of PfamArchitecture objects
    :rettype: list
    """
    name ='?acc='+name if clan else '/'+name
    url='http://pfam.xfam.org/domaingraphics'+name
    try:
        html=urlopen(url)
    except (HTTPError, URLError) as e:
        print(e)
        return []
    else:
        soup=BeautifulSoup(html, 'html.parser')
        architectures=[]
        for h3 in soup.find_all('h3'):
            text=h3.string
            architectures.append(PfamArchitecture(text))
        return architectures



from .xfam_to import *
from .xfam_object import *
from .family_architectures import *
from .hammer_to_object import *
from .hmm_download import *
import os


class PfamFamily(XfamObject):
    """class contains information about pfam family.

    :param access: pfam access of family
    :type access: str
    :param short_name: pfam id of family
    :type short_name: str
    :param type: A type of pfam entry
    :type type: str
    :param seed_len: Number o sequences tin the seed alignment of family
    :type seed_len: int
    :param full_len: Number of sequences in the full alignment of family
    :type full_len: int
    :param avarage_len: Avarage length of amino-acid regions in the full alignment
    :type: float
    :param avarage_id: Avarage percentage identity of sequences in the full alignment
    :type avarage_id: float
    :param avarage_coverage: Fraction of whole sequence length that pfam entry covers
    :type avarage_coverage: float
    :param changestatus: Has family been changes or been added in this pfam realse?
    :type changestatus: str
    :param description: A short description of the family
    :type description: str
    :param go_ref: List of GO ids associated with a family
    :type go_ref: list
    :param so_ref: List of SO ids associated with a family
    :type so_ref: list
    :param pubmed_ref: List of Pubmed ids associated with a family
    :type pubmed_ref: list
    :param pdb_ref: List of PDB names associated with a family
    :type pdb_ref: list
    """

    def __init__(self, family):
        """Constructor method.

        :param family: pfam access or pfam family id
        :type family: str
        """

        super().__init__(family, db='pfam', type_='family')
        if re.match('[0-9]', self.short_name[0]):
            p = 'numbers'
        else:
            p = self.short_name[0].lower()
        try:
            url = 'http://pfam.xfam.org/family/browse?browse=' + p
            html = urlopen(url)
        except (HTTPError, URLError) as e:
            print(e, url)
            return None
        else:
            soup = BeautifulSoup(html, 'html.parser')
            table = soup.find('table', attrs={"class": "details browse"})
            for a in table.find_all('a', href=True):
                if a.string == self.short_name:
                    tr = a.parent.parent
                    columns = tr.find_all('td')
                    self.type = columns[2].string
                    self.seed_len = int(columns[3].string)
                    self.full_len = int(columns[4].string)
                    self.avarage_len = float(columns[5].string)
                    self.avarage_id = float(columns[6].string)
                    self.avarage_coverage = float(columns[7].string)
                    self.changestatus = columns[9].string
                    self.description = columns[10].string
        self.go_ref = pfam_to_go(self.access)
        self.so_ref = pfam_to_so(self.access)
        self.pubmed_ref = pfam_to_pubmed(self.access)
        self.pdb_ref = pfam_to_pdb(self.access)

    def get_full(self):
        """Get full alignemts of PfamFamily sequences in the fasta format.

        :return: Biopython iterator of sequences from alignment
        :rtype: Bio.SeqIO.FastaIO.FastaIterator
        """
        url = 'https://pfam.xfam.org/family/%s' % self.access
        url += '/alignment/full'
        url += '/format?format=fasta&alnType=fasta&order=a&case=l&gaps=dashes&download=1'
        r = requests.get(url, allow_redirects=True)
        r.raise_for_status()
        full = SeqIO.parse(StringIO(r.text), 'fasta')
        return full

    def get_seed(self):
        """Get seed alignemts of PfamFamily sequences in the fasta format.

        :return: Biopython iterator of sequences from alignment
        :rtype: Bio.SeqIO.FastaIO.FastaIterator
        """
        url = 'https://pfam.xfam.org/family/%s' % self.access
        url += '/alignment/seed'
        url += '/format?format=fasta&alnType=fasta&order=a&case=l&gaps=dashes&download=1'
        r = requests.get(url, allow_redirects=True)
        r.raise_for_status()
        seed = SeqIO.parse(StringIO(r.text), 'fasta')
        return seed

    def get_architectures(self):
        """Get a list of architectures of PfamFamily.

        :return: list of PfamArchitecture objects
        :rettype: list
        """
        return family_architectures(self.access)

    def get_hmm(self):
        """ Get the hmm object

        :return: hmm object
        :rtype: hmmer_file.HMMERProfileFileBuilder
        """
        download_hmm([self.access], ".")
        file_name = self.access + ".hmm"
        hmm_object = file_to_object(file_name)
        os.remove(file_name)
        return hmm_object





#!/usr/bin/env python

"""The setup script."""

from setuptools import setup

with open('README.md') as readme_file:
    readme = readme_file.read()
    
requirements = ['beautifulsoup4>=4.9.3', 'biopython>=1.79', 'setuptools>=50.3.0','requests~=2.25.1', 'bs4>=0.0.1']


setup_requirements = ['pytest-runner', ]

test_requirements = ['pytest>=3', ]

setup(
    author="Julia Gołębiowska, Adam Cicherski, Patrycja Owczarek",
    author_email='je.golebiowska@student.uw.edu.pl',
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="Project which enable automatic downloading data from pfam and rfam databases",
    entry_points={
        'console_scripts': [
            'PARS=PARS.cli:main',
        ],
    },
    install_requires=requirements,
    license="MIT license",
    long_description=readme,
    include_package_data=True,
    keywords='PARS',
    name='PARS',
    packages=['pars'],
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/JuliaGol/PARS',
    project_urls={"Bug Tracker":"https://github.com/JuliaGol/PARS/issues"},
    version='0.1.4',
    zip_safe=False,
)



class PfamArchitecture():
    """Simple class for protein domains architecture.
    
    :param domains: list of domains in the architecture
    :type domains: list
    :param number_of_sequences: number of sequences with this architecture
    :type number_of_sequences: int
    """
    def __init__(self,text):
        self.domains=text.split(':')[-1].split(', ')
        self.domains=[domain.replace(' ','') for domain in self.domains]
        self.number_of_sequences=int("".join(filter(str.isdigit, text)))
        
    def __str__(self):
        return '(Architecture '+str(self.domains)+','+'number of sequences '+str(self.number_of_sequences)+')'
    
    def __repr__(self):
        return str(type(self).__name__)+str((self.domains,self.number_of_sequences))
